/*
	d_sky.c

	$Id: d_sky.c,v 1.3 2007/03/14 21:03:06 sezero Exp $
*/

#include "quakedef.h"
#include "r_local.h"
#include "d_local.h"

#define SKY_SPAN_SHIFT	5
#define SKY_SPAN_MAX	(1 << SKY_SPAN_SHIFT)


/*
=================
D_Sky_uv_To_st
=================
*/
static void D_Sky_uv_To_st (int u, int v, fixed16_t *s, fixed16_t *t){
	float	wu, wv, temp, temp2;
	vec3_t	end;

	if (r_refdef.vrect.width >= r_refdef.vrect.height){
		temp = 1/(float)r_refdef.vrect.width;
        }
	else{temp = 1/(float)r_refdef.vrect.height;}

	wu = 8192.0 * (float)(u - ((int)vid.width >> 1)) * temp;
	wv = 8192.0 * (float)(((int)vid.height >> 1) - v) * temp;

	end[0] = 4096*vpn[0] + wu*vright[0] + wv*vup[0];
	end[1] = 4096*vpn[1] + wu*vright[1] + wv*vup[1];
	end[2] = (4096*vpn[2] + wu*vright[2] + wv*vup[2]) * 3;
	VectorNormalize (end);

	temp = skytime*skyspeed;	// TODO: add D_SetupFrame & set this there
	temp2 = 3*SKYSIZE - 6;
	*s = (int)((temp + temp2 * end[0]) * 0x10000);
	*t = (int)((temp + temp2 * end[1]) * 0x10000);
    }


/*
=================
D_DrawSkyScans8
=================
*/
void D_DrawSkyScans8 (espan_t *pspan){
	int				count, spancount, u, v;
	unsigned char	*pdest;
	fixed16_t		s, t, snext, tnext, sstep, tstep;
	int				spancountminus1;

	sstep = 0;	// keep compiler happy
	tstep = 0;	// ditto

	do{
		pdest = (unsigned char *)((byte *)d_viewbuffer +(screenwidth * pspan->v) + pspan->u);
		count = pspan->count;

	// calculate the initial s & t
		u = pspan->u;
		v = pspan->v;
		D_Sky_uv_To_st (u, v, &s, &t);

		do{
			if (count > SKY_SPAN_MAX){
			    spancount = SKY_SPAN_MAX;
			    count -= spancount;
                u += spancount;
                D_Sky_uv_To_st (u, v, &snext, &tnext);
				sstep = (snext - s) >> SKY_SPAN_SHIFT;
				tstep = (tnext - t) >> SKY_SPAN_SHIFT;
                }
			else{
			    spancount = count;
			    count = 0;
				spancountminus1 = (float)(spancount - 1);
				if (spancountminus1){
					u += spancountminus1;
					D_Sky_uv_To_st (u, v, &snext, &tnext);

					sstep = (snext - s) / spancountminus1;
					tstep = (tnext - t) / spancountminus1;
                    }
                }

			do  {
				*pdest++ = r_skysource[((t & R_SKY_TMASK) >> 8) + ((s & R_SKY_SMASK) >> 16)];
				s += sstep;
				t += tstep;
                } while (--spancount);
			s = snext;
			t = tnext;
		} while (count);
	} while ((pspan = pspan->pnext));
}

